"use strict";
/*
 * Copyright (c) 2020 Broadcom.
 * The term "Broadcom" refers to Broadcom Inc. and/or its subsidiaries.
 *
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *   Broadcom, Inc. - initial API and implementation
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.retrieveWithDependencies = void 0;
const vscode = require("vscode");
const Element_1 = require("../model/Element");
const globals_1 = require("../globals");
const RETRIEVE_ELEMENTS_LIMIT = 20;
function retrieveWithDependencies(arg, retrieveElementService) {
    vscode.window.withProgress({
        cancellable: true,
        location: vscode.ProgressLocation.Notification,
        title: "Retrieving",
    }, (progress, token) => __awaiter(this, void 0, void 0, function* () {
        token.onCancellationRequested(() => {
            globals_1.logger.info("Retrieve Cancelled.");
        });
        if (!(vscode.workspace.workspaceFolders && vscode.workspace.workspaceFolders.length > 0)) {
            globals_1.logger.error("Specify workspace before retrieving elements");
            return;
        }
        const workspace = vscode.workspace.workspaceFolders[0];
        const repo = arg.getRepository();
        const eq = arg.getQualifier();
        progress.report({ increment: 0, message: "Dependencies List" });
        const elementsToRetrieve = yield retrieveElementService.retrieveDependenciesList(repo, eq);
        if (yield hitLimit(elementsToRetrieve, eq)) {
            return;
        }
        if (elementsToRetrieve.length === 0) {
            elementsToRetrieve.push(createElementFromQualifier(repo, eq));
        }
        const incrementNumber = 100 / (elementsToRetrieve.length + 1);
        // retrieve dependencies
        let firstOpened = false;
        for (let i = 0; i < elementsToRetrieve.length; i++) {
            if (token.isCancellationRequested) {
                return;
            }
            const elName = elementsToRetrieve[i].elmName;
            try {
                progress.report({ message: "(" + (i + 1) + "/" + elementsToRetrieve.length + ") " + elName });
                const eQualifier = createElementQualifier(elementsToRetrieve[i]);
                const filePath = yield retrieveElementService.retrieveElement(workspace, repo, elName, eQualifier);
                if (!firstOpened) {
                    const doc = yield vscode.workspace.openTextDocument(filePath);
                    vscode.window.showTextDocument(doc, { preview: false });
                }
            }
            catch (error) {
                retrieveElementService.processRetrieveElementError(error);
            }
            finally {
                // If first with error, don't open anything
                firstOpened = true;
                progress.report({
                    increment: incrementNumber,
                    message: "(" + (i + 1) + "/" + elementsToRetrieve.length + ") " + elName,
                });
            }
        }
    }));
}
exports.retrieveWithDependencies = retrieveWithDependencies;
/**
 * Only to be used in the scope of this module.
 * Creates the Element Qualifier for the dependencies.
 * @param element
 */
function createElementQualifier(element) {
    // tslint:disable-next-line: prefer-immediate-return
    const eQualifier = {
        element: element.elmName,
        env: element.envName,
        stage: element.stgNum,
        subsystem: element.sbsName,
        system: element.sysName,
        type: element.typeName,
    };
    return eQualifier;
}
/**
 * Only to be used in the scope of this module.
 * Creates an element based on repository and element qualifier.
 * @param repo
 * @param eq
 */
function createElementFromQualifier(repo, eq) {
    const iElement = {
        elmName: eq.element,
        envName: eq.env,
        sysName: eq.system,
        // tslint:disable-next-line:object-literal-sort-keys
        sbsName: eq.subsystem,
        stgNum: eq.stage,
        typeName: eq.type,
        fullElmName: eq.element,
        elmVVLL: "",
    };
    return new Element_1.Element(repo, iElement);
}
function hitLimit(elementsToRetrieve, eq) {
    return __awaiter(this, void 0, void 0, function* () {
        if (elementsToRetrieve.length <= RETRIEVE_ELEMENTS_LIMIT) {
            return false;
        }
        const msg = `Element ${eq.element} has ${elementsToRetrieve.length} dependencies.`;
        return (yield vscode.window.showWarningMessage(msg, "Download all", "Cancel")) === "Cancel";
    });
}
//# sourceMappingURL=RetrieveElementWithDependencies.js.map