"use strict";
/*
 * Copyright (c) 2020 Broadcom.
 * The term "Broadcom" refers to Broadcom Inc. and/or its subsidiaries.
 *
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *   Broadcom, Inc. - initial API and implementation
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Repository = void 0;
const EndevorEntity_1 = require("./EndevorEntity");
const EndevorFilter_1 = require("./EndevorFilter");
class Repository extends EndevorEntity_1.EndevorEntity {
    constructor(name, url, username, password, datasource, profileLabel, id) {
        super();
        this._id = id;
        this.name = name;
        this.url = url;
        this.username = username;
        this.password = password;
        this.datasource = datasource;
        this._environments = new Map();
        this._filters = [];
        this._map = new EndevorFilter_1.EndevorFilter(this, EndevorFilter_1.FILTER_ALL_STRING);
        this._profileLabel = profileLabel;
    }
    loadInfoFromConfig(repo) {
        this.name = repo.name;
        this.url = repo.url;
        this.username = repo.username;
        this.datasource = repo.datasource;
        this._filters = repo.filters;
        this._filters.forEach(filter => {
            filter.setRepository(this);
        });
    }
    loadEnvironments(envs, append) {
        if (!append) {
            this._environments = new Map();
        }
        envs.forEach(env => {
            env.repository = this;
            this._environments.set(env.envName, env);
        });
    }
    findEnvironment(envName) {
        return this._environments.get(envName);
    }
    findSystem(envName, sysName) {
        const env = this.findEnvironment(envName);
        if (env) {
            return env.findSystem(sysName);
        }
    }
    findType(typeName, envName, sysName) {
        const system = this.findSystem(envName, sysName);
        if (system) {
            return system.findType(typeName);
        }
    }
    findFilter(uri) {
        for (const filter of this.filters) {
            if (filter.getUri() === uri) {
                return filter;
            }
        }
    }
    get id() {
        return this._id;
    }
    set id(value) {
        this._id = value;
    }
    setName(value) {
        this.name = value;
    }
    getName() {
        return this.name;
    }
    setProfileLabel(value) {
        this._profileLabel = value;
    }
    getProfileLabel() {
        return this._profileLabel;
    }
    isAttachedToConnection(value) {
        return value === this._profileLabel;
    }
    setUsername(value) {
        this.username = value;
    }
    getUsername() {
        return this.username;
    }
    setPassword(value) {
        this.password = value;
    }
    getPassword() {
        return this.password;
    }
    getDescription() {
        return this.url + " | " + this.datasource;
    }
    getUrl() {
        return this.url;
    }
    setUrl(value) {
        this.url = value;
    }
    getDatasource() {
        return this.datasource;
    }
    setDatasource(value) {
        this.datasource = value;
    }
    getEnvironmentMap() {
        return this._environments;
    }
    get environments() {
        return Array.from(this._environments.values());
    }
    get filters() {
        return this._filters;
    }
    set filters(value) {
        this._filters = value;
    }
    get map() {
        return this._map;
    }
    set map(value) {
        this._map = value;
    }
    getUrlString() {
        let urlPath = "EndevorService/rest/" + this.datasource;
        if (this.datasource !== "") {
            urlPath = urlPath + "/";
        }
        if (!this.url.endsWith("/")) {
            urlPath = "/" + urlPath;
        }
        return this.url + urlPath;
    }
    getRepository() {
        return this;
    }
    getIFilters() {
        const resultFilters = [];
        this._filters.forEach(filter => {
            resultFilters.push({ uri: filter.getUri() });
        });
        return resultFilters;
    }
    /**
     * Checks if this repository is the same instance (url, datasource and username are equal) as parameter `repo`
     * @param repo Repository to compare with this repository
     * @return True if this repository and `repo` have the same Endevor instance.
     */
    isSameInstance(repo) {
        if (!repo) {
            return false;
        }
        if (this.url !== repo.getUrl()) {
            return false;
        }
        if (this.datasource !== repo.getDatasource()) {
            return false;
        }
        if (this.username !== repo.getUsername()) {
            return false;
        }
        return true;
    }
    /**
     * Checks if this repository is the same as parameter `repo`. Password and metadata are not used in the comparison.
     * @param repo Repository to compare with this repository
     * @return True if this repository and `repo` are the same.
     */
    isEqual(repo) {
        if (!repo || !this.isSameInstance(repo)) {
            return false;
        }
        if (repo.getName() !== this.getName()) {
            return false;
        }
        if (repo.filters.length !== this.filters.length) {
            return false;
        }
        for (const filter of repo.filters) {
            if (!this.findFilter(filter.getUri())) {
                return false;
            }
        }
        return true;
    }
}
exports.Repository = Repository;
//# sourceMappingURL=Repository.js.map